;
; LCD


	list	p=18f4550
	#include "p18f4550.inc"


	org 200			; bootloader reloc. reset vector
	goto	start


; -----------------

LCD			equ		PORTB
LCDTRIS		equ		TRISB
LCDC		equ		PORTA
LCDCTRIS	equ		TRISA

XCLK		equ	0
YCLK		equ	1
DIN			equ	2

; -----------------

	org H'220' 		; bootloader reloc. main
#include "string_m.inc"


start:

	bcf		WDTCON, SWDTEN		; kill watchdog timer

	;movlw	B'10000000'			; INTOSC 32KHz (test), prefer external
	movlw	B'11110000'			; INTOSC 8MHz, prefer external
	movwf	OSCCON


	MOVLW 0Fh 					; Configure A/D
	MOVWF ADCON1 				; for digital inputs
	MOVLW 07h 					; Configure comparators
	MOVWF CMCON 				; for digital input


; --- init_ports
	clrf	LCDTRIS
	clrf	LCD		; 0s for data out

	bcf		LCDCTRIS, XCLK
	bcf		LCDCTRIS, YCLK
	bcf		LCDCTRIS, DIN
	clrf	LCDC
; ---


	call	INITUSART


	sendstring	str_ready



main:

	setf	LCD			; dummy data
	call	lineclk		; initial line clock before data start
sendscreen:
	;call	setup_data	; test pattern

	movlw	upper	image1		; point to start of imge data...
	movwf	TBLPTRU
	movlw	high	image1
	movwf	TBLPTRH
	movlw	low		image1
	movwf	TBLPTRL	

	bsf		LCDC, DIN	; first line has DIN sync for its entire duration...
	call	sendline	; ...
	bcf		LCDC, DIN	; after next line clock, but before 2nd line

	movlw	H'ef'		; 239 lines to be counted (does not include 1st line and 2 dummy lines at end)
	movwf	LINECNTL	; ...


sendscreen2:
	call	sendline
	; --- test hack
	;movlw	H'C0'
	;xorwf	LINECNTL, w
	;bnz		hackjump

	;movlw	upper	image1		; point to start of imge data...
	;movwf	TBLPTRU
	;movlw	high	image1
	;movwf	TBLPTRH
	;movlw	low		image1
	;movwf	TBLPTRL	

	; --- end test hack
hackjump:
	decfsz	LINECNTL	; done sending lines?
	goto	sendscreen2	; if no

	;setf	LCD			; dummy HIGH data for last 2 lines
	call	senddummyline
	call	senddummyline


	goto	sendscreen



clk:
	TBLRD*+				; data from table...
	movf	TABLAT, w	;
	movwf	LCD			; to LCD data port

	bsf		LCDC, XCLK
	;nop
	bcf		LCDC, XCLK
	return

lineclk:
	bsf		LCDC, YCLK

	call	clk

	bcf		LCDC, YCLK
	return


sendline:

	movlw	H'9F'				; 160 bytes per line... 159 pulses the 'normal' way
	movwf	BYTECNTL			; high not used
sendline2:
	call	clk
	;bsf		LCDC, XCLK
	;nop
	;nop
	;nop
	;nop
	;nop
	;nop
	;nop
	;nop
	;bcf		LCDC, XCLK

	decfsz	BYTECNTL			; sent all bytes of line?
	goto	sendline2			; if no
	call	lineclk				; last byte of line is also a line clock

	return

senddummyline:

	movlw	H'9F'				; 160 bytes per line... 159 pulses the 'normal' way
	movwf	BYTECNTL			; high not used
senddummyline2:
	;call	clk

	movlw	H'FF'		; match timing of 'call'
	movwf	LCD

	nop
	nop
	nop
	nop


	bsf		LCDC, XCLK
	bcf		LCDC, XCLK


	decfsz	BYTECNTL			; sent all bytes of line?
	goto	senddummyline2		; if no
								; last byte of line is also a line clock
	bsf		LCDC, YCLK

	bsf		LCDC, XCLK			; dummy clock
	bcf		LCDC, XCLK

	bcf		LCDC, YCLK

	return


setup_data:
	movlw	B'00111100'			; dummy data...
	movwf	LCD
	return



#include "4220uart.inc"
#include "vars.inc"
#include "image1.inc"			; actual image data


; ------
str_ready:
	DW	"Hello World! Sending things to Earth....\r\n\x00"

	end
